-- The 'storage' module provides the ability to store data that is specific to a plugin.
-- Each plugin has its own private storage in a key-value table.
-- This storage keeps a record of persistent private data like credentials, device IP address, MAC address, etc.

-- Lua storage module documentation is available at https://developer.mios.com/api/scripting/modules/storage/

local _storage = require("storage")

-- The 'logging' module handles log processing and supports different log severities, indents, etc.
-- Users can change the global log severity on our HUB and can modify log verbosity levels. 
-- More verbose logs are helpful when debugging issues.

-- The Logger API, which lets you change hub log level, is at https://developer.mios.com/api/hub/system/logging/hub-log-local-set/

-- Lua logging module documentation is available at https://developer.mios.com/api/scripting/modules/logging/

local _logger = require("logging")

-- We declare module level variables in UPPERCASE LETTERS to better distinguish their scope:

local STORAGE_ACCOUNT_KEY = _G.constants.STORAGE_ACCOUNT_KEY

-----------------------------------------------------------
-- Any parameter passed to the script can be obtained by using the vararg expression "..."
-- See https://www.lua.org/manual/5.4/manual.html#3.4.11
-- We store the parameter in the "args" variable for later use (while also making sure to initialize it if no parameters are passed):

local args = ... or {}

-- Here, we declare a table that will hold the new credentials after we check them:

local account_credentials = {}

if type(args.username) == "string" and args.username ~= "" then
    account_credentials.username = args.username
end

if type(args.password) == "string" and args.password ~= "" then
    account_credentials.password = args.password
end

-- The following simulates a valid login if we have a new credential set:
if account_credentials.username == _G.constants.DEFAULT_USERNAME and account_credentials.password == _G.constants.DEFAULT_PASSWORD then

    -- '_logger.info' will save the logs on the HUB at: /var/log/firmware/ha-luad.log
    -- Example: 2021-12-13 19:32:11.863117 INFO : prefix.node_test_plugin: Logged in successfully...
    -- Lua logging module documentation is available at https://developer.mios.com/api/scripting/modules/logging/
    
    _logger.info("Logged in successfully...")

    -- '_storage.set_table' will save the table in the plugin's own storage under the key specified by the   
    --  'STORAGE_ACCOUNT_KEY' variable.
    
    -- Storage module documentation is available at https://developer.mios.com/api/scripting/modules/storage/
    
    _storage.set_table(STORAGE_ACCOUNT_KEY, account_credentials)

    -- The 'loadfile' function runs a script located at a specific path and returns a
    -- function that we call to execute the script.
    
    -- NOTE - The path must begin with the identifier "HUB:"

    -- Our lua interpreter is modified so that when calling 'loadfile' it will replace the "HUB:" identifier
    -- with the real path to the plugin directory on the controller. For example:
    
    -- Official plugins directory is: /opt/firmware/plugins/ (subject to change)
    
    -- Custom plugins ("HUB:" plugins) directory is: /home/data/custom_plugins (also subject to change)

    -- After we've successfully authenticated we begin the process of adding the second device:
    
    loadfile("HUB:prefix.node_test_plugin/scripts/functions/create_device")()
else
    -- Or else, log a warning and finish
    
    -- '_logger.warning' will save the logs on the HUB at: /var/log/firmware/ha-luad.log
    
    -- For example: 2021-12-13 19:32:11.863117 WARN : prefix.node_test_plugin: Missing device ID, can't delete device
    
    -- Lua logging module documentation is available at https://developer.mios.com/api/scripting/modules/logging/
    
    -- 'Warning' severity logs are bigger than 'Info' logs, so these are always written in the ha-luad.log file.
    
    _logger.warning("The provided credentials are invalid.")
end
