-- The 'core' module provides the ability to work with the Ezlo Lua API to manage devices, items, settings and more.
-- Lua core module documentation is available at: https://developer.mios.com/api/scripting/modules/core/functionality/

local _core = require("core")

-- The 'logging' module handles log processing and supports different log severities, indents, etc.
-- Users can change the global log severity on our HUB and can modify log verbosity levels. 
-- More verbose logs are helpful when debugging issues.

-- The Logger API, which lets you change hub log level, is at https://developer.mios.com/api/hub/system/logging/hub-log-local-set/

-- Lua logging module documentation is available at https://developer.mios.com/api/scripting/modules/logging/

local _logger = require("logging")

-- Any parameter passed to the script can be obtained by using the vararg expression "..."
-- See https://www.lua.org/manual/5.4/manual.html#3.4.11
-- We store the parameter in the "args" variable for later use (while also making sure to initialize it if no parameters are passed).

-- Example payload passed to this script by the firmware when the user initiates a device removal:
--[[
    {
        deviceId: "61b1f9420000001241d7831c"
    }
]]
local params = ... or {}

-- Now we make a sanity check to see if this script was called correctly and a device ID was provided:

if not params.deviceId then
    -- '_logger.warning' will save the logs on the HUB at: /var/log/firmware/ha-luad.log
    -- Example: 2021-12-13 19:32:11.863117 WARN : prefix.node_test_plugin: Missing device ID, can't delete device
    
    -- Lua logging module documentation is available at https://developer.mios.com/api/scripting/modules/logging/
    -- 'Warning' severity severity logs are bigger than 'Info' logs, so these are always written 
    -- in the ha-luad.log file.
    
    _logger.warning("Missing device ID, can't delete device")
    return
end

-- Next, log the device deletion at the INFO level for debugging purposes.
-- This is an important lifecycle event which must be logged always.

_logger.info("Delete device: " .. params.deviceId)

-- Now, we attempt to remove the device by its ID:

_core.remove_device(params.deviceId)

--             NOTES and guidelines for the delete_device.lua script

-- * This script is configured in the config.json file like this:

--[[
{
    "gateway": 
    {
        "forceRemoveDeviceCommand": "HUB:prefix.node_test_plugin/scripts/delete_device",
    }
}       ]]

-- * You can configure any script path as your device remover or use this as a template
--   Choosing not to configure this script means that users cannot remove
--   individual devices that the plugin creates EXCEPT when uninstalling the plugin. In the latter case,
--   *all* devices created by the plugin are removed.

-- * API Tool - https://apitool.ezlo.com/dashboard
--    This tool lets you simulate the requests made by our mobile and web apps so 
--    you can test and debug your plugins.
--    You can use this tool to call your items, devices, device setting and even your individual
--    scripts without having to use the mobile or web apps.

--    Example request with the API tool to test your device removal script without using a mobile or web app
--   (you need to provide the correct device ID, of course):
--[[
    {
        "method": "hub.device.force_remove",
        "id": "_AUTO_738951",
        "params": {
            "_id": "61a0f524123e3309fc77ae7b"
        }
    }
]]
