-- This script lets you call an API function to create a device on your plugin. You can also use it to specify device type, category, id, battery requirements etc. The script contains a call to core.add_device in the core api module.

-- Note - ‘create_device.lua’ is just our name for the script in our example. You can name it however you please.

-- The 'core' module provides the ability to work with the Ezlo Lua API to manage devices, items, settings and more.
-- Lua core module documentation is available at: https://developer.mios.com/api/scripting/modules/core/functionality/

local _core = require("core")

-- The 'logging' module handles log processing and supports different log severities, indents, etc.
-- Users can change the global log severity on our HUB and can modify log verbosity levels. 
-- More verbose logs are helpful when debugging issues.

-- The Logger API, which lets you change hub log level, is at https://developer.mios.com/api/hub/system/logging/hub-log-local-set/

-- Lua log module documentation is available at https://developer.mios.com/api/scripting/modules/logging/

local _logger = require("logging")

-- The 'storage' module provides the ability to store data that is specific to a plugin.
-- Each plugin has its own private storage in a key-value table.
-- This storage keeps a record of persistent private data like credentials, device IP address, MAC address, etc.

-- Lua storage module documentation is available at https://developer.mios.com/api/scripting/modules/storage/

local _storage = require("storage")

--------------------------------------------------------------------
-- Here we access the '_G.constants' table that we initialized in our startup.lua script.
-- We also retrieve the credentials saved in our plugin storage table by checking a 'known' key:

local credentials = _storage.get_table(_G.constants.STORAGE_ACCOUNT_KEY)

-- No credentials means that the user did not log in so we display a warning:

if not credentials then
    _logger.warning("No accout is configured... The user did not log in yet.")
    -- return
end
--------------------------------------------------------------------

-- Here we define a function that handles the logic for creating a device.
-- NOTE - the function is defined here but is actually called at the end of the script.

local function CreateDevice ()
    
    -- First we try to get the gateway/plugin ID:
    
    local my_gateway_id = (_core.get_gateway() or {}).id

    -- If we cannot get the gateway/plugin ID then just log an error and return. There's nothing else we can do.
    
    if not my_gateway_id then
        
        -- This should never happen, but it's better to be on the safe side.
        
        _logger.error("Failed to get current gateway ID, skip creating devices")
        return nil
    end

    local count = 0
    
    -- Next we enumerate all devices in our controller:
    
    for _, device in pairs(_core.get_devices() or {}) do
        
        -- ...and filter out devices that don't belong to our gateway/plugin:
        
        if device.gateway_id == my_gateway_id then
            
            -- We now count how many devices we have created:
            
            count = count + 1
            
            -- Our objective is to create a fake switch device if the user installed the plugin, and create
            -- a second device after the user has logged in. 
            -- So we check how many devices we have and if the user is logged in:
            
            if not credentials or count >=2 then
                
                -- This means that if a device is found but the user isn't authenticated, we still return the
                -- device.id to signal that we have a device and don't need another one.
                -- But if the user is authenticated (we found his credentials), then we check the device count. If it is 1 then
                -- we don't return which means a new device will be created below after the loop ends.
                
                return device.id
            end
        end
    end

    _logger.info("Create new fake device")

    -- '_core.add_device' is used to create devices on Ezlo controllers.
    -- It returns the ID of any newly created device.
    
    -- Device add documentation: https://developer.mios.com/api/scripting/modules/core/functionality/add_device/
    -- Device Object documentation: https://developer.mios.com/api/scripting/modules/core/objects/device/
    
    return _core.add_device {
        type = "switch.inwall",
        device_type_id = "switch.inwall.fake",
        category = "switch",
        subcategory = "interior_plugin",
        battery_powered = false,
        gateway_id = _core.get_gateway().id,
        
        -- Here, we just slightly modify the device name to see the difference between them:
        
        name = not credentials and "Fake Switch (install)" or "Fake Switch (login)",
        info = {
            manufacturer = "Ezlo",
            model = "1.0"
        }
    }
end

-- Here we define a function that handles the logic to create an item.
-- NOTE - the function is defined here but is actually called at the end of the script.

local function CreateItem (device_id)
    
    -- 'items' are associated with devices, so if we don't have a device_id then we cannot create the item:
    
    if not device_id then
        
        -- ...log an error:
        
        _logger.error("Cannot create item. Missing device_id...")
        
        -- and return control
        
        return
    end

    -- '_core.get_items_by_device_id(device_id)' will only return the items of a specific device.
    
    -- Documentation: https://developer.mios.com/api/scripting/modules/core/functionality/get_items_by_device_id/
    
    for _, item in ipairs(_core.get_items_by_device_id(device_id) or {}) do
        
        -- Here we check whether the device already has items. If so, the control will reach
        -- this block and simply return. We are not interested in creating duplicate items
        -- and this is a simple way of preventing it.
        
        return
    end

    -- Log a message at 'info' severity level since this is an important lifecycle message:
    
    _logger.info("Create new fake 'switch' item")

    -- '_core.add_item' is used to create items on Ezlo controllers.
    -- It returns the ID of any newly created item.
    
    -- Item add documentation: https://developer.mios.com/api/scripting/modules/core/functionality/add_item/
    
    -- Item object documentation: https://developer.mios.com/api/scripting/modules/core/objects/item/
    
    _core.add_item({
        device_id = device_id,
        name = "switch",
        value_type = "bool",
        value = false,
        has_getter = true,
        has_setter = true
        
    -- has_setter - Capabilities which can be set are exposed as ‘Actions’ in meshbots. This is defined by the has_setter field of the item object. 

       -- You can view a list of setters at https://developer.mios.com/api/scripting/modules/core/functionality/set_setting_value/

    -- has_getter - Capabilities which can be read are exposed as ‘Triggers’ in meshbots. This is defined by the has_getter field of the item object. 

       -- You can view a list of getters at https://developer.mios.com/api/scripting/modules/core/functionality/getters/
        
    })
end

--------------------------------------------------------------------

-- We've put the 'business logic' at the end of the script because the Lua language forces us to define functions before calling them.

-- Every time this script is called, we want to try and create a device and its corresponding item:

local device_id = CreateDevice()

-- Next, create the item for our new device. The cases where 'device_id'is nil are handled inside the 'CreateItem' function:

CreateItem(device_id)
