-- The 'core' module provides the ability to work with the Ezlo Lua API to manage devices, items, settings and more.
-- Lua core module documentation is available at: https://developer.mios.com/api/scripting/modules/core/functionality/

local _core = require("core")

-- The 'logging' module handles log processing and supports different log severities, indents, etc.
-- Users can change the global log severity on our HUB and can modify log verbosity levels. 
-- More verbose logs are helpful when debugging issues.

-- The Logger API, which lets you change hub log level, is at https://developer.mios.com/api/hub/system/logging/hub-log-local-set/

-- Lua log module documentation is available at https://developer.mios.com/api/scripting/modules/logging/

local _logger = require("logging")


-- Any parameter passed to the script can be obtained by using the vararg expression "..."
-- See https://www.lua.org/manual/5.4/manual.html#3.4.11
-- We store the parameter in the "args" variable for later use (while also making sure to initialize it if no parameters are passed):

local params = ... or {}

-- There are 3 ways you can call this script:
-- 1. https://developer.mios.com/api/hub/items/api/hub-item-value-set-single-item/
-- 2. https://developer.mios.com/api/hub/items/api/hub-item-value-set-multiply-items-with-one-value/
-- 3. https://developer.mios.com/api/hub/items/api/hub-item-value-set-different-items-with-different-values/

-- The 'if' block shown below converts the first 2 syntaxes to the third so that we support all types of calls to this script.

-- Developers can chose to support only the first (set-single-item) which means the other 2 will return an error.
-- Currently the mobile and web apps only use the first approach (set-single-item), but this may change in the future.
-- Options #2 and #3 are meant to optimize commands for a batch of devices (e.g. 'turn off all lights'), or a
-- combination of different commands for multiple device types in a single web socket message.

local items = {}

if type(params.item_id) == "string" then
    -- Single item and single value.
    -- Create a map with a single item and its value.
    --[[ Example, the structure of the params payload for option #1 is as follows:
    {
        "item_id": "619634c0123e33191314af83",
        "value": true
    }          ]]
    items = { [params.item_id] = { value = params.value } }

elseif type(params.item_ids) == "table" then
    -- Multiple items and single value.
    -- Create a map with multiple items where each item has the same value.
    --[[ Example, the structure of the params payload for option #2 is as follows:
    {
        "item_ids": ["619634c0123e33191314af83", "619634c0123e33191314af84"]
        "value": true
    }          ]]
    for _, item_id in ipairs(params.item_ids) do
        items[item_id] = { value = params.value }
    end

elseif type(params.items) == "table" then
    -- Multiple items with different values.
    -- Just save it as is.
    --[[ Example, the structure of the params payload for option #3 is as follows:
    {
        "items": {
            "619634c0123e33191314af83": {
                "value": {
                    "value": true
                }
            }
        }
    }          ]]
    items = params.items
end

-- At this point we have a map of items with (possibly diferent) values.
-- Loop each item in the "items" table and execute the script for it.
-- NOTE: In our case we have only 1 item (switch) but this pattern can
-- apply to any arbitrary number of items if you chose to keep this plugin structure
for item_id, item_value in pairs(items) do
    
    -- We only have the item ID's so we need to get the full item info.
    -- Item info details are at: https://developer.mios.com/api/scripting/modules/core/objects/item/
    
    local item = _core.get_item(item_id)

    -- Load the script that holds the item implementation for our device.
    
    -- We chose to create scripts that encapsulate all the item logic inside them
    -- and we named the scripts the same as the item name to be more intuitive.
    
    -- If all items are placed in the same directory we can have a centralized handling
    -- mechanism like this one which just ditributes the requests accordingly:
    local item_script = loadfile("HUB:prefix.node_test_plugin/scripts/items/" .. item.name)

    -- If the script is loaded (which means it exists and the item is implemented)
    if item_script then
        -- Execute the script in protected mode with 'pcall' to catch errors, passing many details
        -- (id of device, item, value, etc) to the item scripts to aid in processing the request.
        
        -- We use 'pcall' because we don't want errors in any item handler to stop execution of the other
        -- item handlers (when there are multiple items that must be changed).
        
        local success, errmsg = pcall(item_script, {
                device_id = item.device_id,
                item_id = item_id,
                value = item_value.value,
                source = params.source
            })
        if not success then
            -- '_logger.error' will save the logs on the HUB at: /var/log/firmware/ha-luad.log
            -- Example: 2021-12-13 19:32:11.863117 ERROR: prefix.node_test_plugin: Failed to set item switch
            
            -- Lua logging module documentation is available at https://developer.mios.com/api/scripting/modules/logging/
            -- 'Error' severity logs are bigger than 'Info' logs, so these are always written in the ha-luad.log file.
            
            _logger.error("Failed to set item: " .. item.name .. (errmsg and (", error: " .. errmsg) or ""))
        end
    else
        -- '_logger.error' will save the logs on the HUB at: /var/log/firmware/ha-luad.log
        -- Example: 2021-12-13 19:32:11.863117 ERROR: prefix.node_test_plugin: Failed to load handler for item: switch
        
        -- Lua logging module documentation: https://api.ezlo.com/scripting/logging/index.html
        -- 'Error' severity logs are bigger than 'Info' logs, so these are always written in the ha-luad.log file.
        _logger.error("Failed to load handler for item: " .. item.name)
    end
end

--            NOTES and guidelines for the set_item_value.lua script

-- * This script is configured in the config.json file like this:

--[[
{
    "gateway":
    {
        "setItemValueCommand": "HUB:prefix.node_test_plugin/scripts/set_item_value",
    }
}        ]]

-- * You can configure any script path as your item handler or use this as a template.
--   Choosing not to configure this script means that users will not be able to
--   set or modify any item the plugin creates.

-- * This script is called automatically when making any of the following requests from the web socket API tool:
-- 1. https://developer.mios.com/api/hub/items/api/hub-item-value-set-single-item/
-- 2. https://developer.mios.com/api/hub/items/api/hub-item-value-set-multiply-items-with-one-value/
-- 3. https://developer.mios.com/api/hub/items/api/hub-item-value-set-different-items-with-different-values/

-- * API Tool - https://apitool.ezlo.com/dashboard
--    This tool lets you simulate the requests made by our mobile and web apps so 
--    you can test and debug your plugins.
--    You can use this tool to call your items, devices, device setting and even your individual
--    scripts without having to use the mobile or web apps.

--   Example API request to call any script from the API tool:
--[[
    {
        "method": "hub.extensions.plugin.run",
        "id": "_ID_",
        "params": {
            "script": "HUB:prefix.node_test_plugin/configs/account",
            "scriptParams": {
                "username": "demo123",
                "password": "123demo"
            }
        }
    }
]]
--   This will perform authentication then start creating a device.
--   This could be an API call made from a DPW (device pairing wizard) as part of the plugin
--   config. to authenticate your plugin so it works properly.
