-- 'startup.lua' is a script which is called every time the firmware is started or rebooted. 

-- Among other things, this example of 'startup.lua' calls ‘create_device.lua’ which contains script which
-- lets you create a device in the EZlogic interface. 

-- The 'logging' module handles log processing and supports different log severities, indents, etc.
-- Users can change the global log severity on our HUB and can modify log verbosity levels. 
-- More verbose logs are helpful when debugging issues.

-- The Logger API, which lets you change hub log level, is at https://developer.mios.com/api/hub/system/logging/hub-log-local-set/

-- Lua logging module documentation is available at https://developer.mios.com/api/scripting/modules/logging/

local _logger = require("logging")

-- '_logger.info' will save the logs on the HUB at: /var/log/firmware/ha-luad.log
-- Example: 2021-12-13 19:32:11.863117 INFO : prefix.node_test_plugin: prefix.node_test_plugin starting up...

-- Lua logging module documentation is available at https://developer.mios.com/api/scripting/modules/logging/

_logger.info("prefix.node_test_plugin starting up...")


-- Because the 'constants.lua' file returns a table, we can use 'require' to directly load
-- and return the result of the script (which is a table with constants):

local _constants = require("HUB:prefix.node_test_plugin/configs/constants")

-- Now we initialize a global object that holds constants which are used throughout the plugin execution.

-- We can use this approach because we have set the following configuration in the config.json file:
--  "executionPolicy": "restoreLastScriptState"

-- This enables us to use the same environment across each script execution. All global variables are saved and restored this way.

-- Be careful when using this approach as it can easily lead to memory leaks if not cleared properly after you are done with it!

_G.constants = _constants or {}


--  The 'loadfile' function runs a script located at a specific path and returns a
-- function that we call to execute the script.

-- NOTE - The path must begin with the identifier "HUB:"

-- Our lua interpreter is modified so that when calling 'loadfile' it will replace the "HUB:" identifier
-- with the real path to the plugin directory on the controller. For example:
    
-- Official plugins directory is: /opt/firmware/plugins/ (subject to change)
    
-- Custom plugins ("HUB:" plugins) directory is: /home/data/custom_plugins (also subject to change)

-- The following line loads create_device.lua. This script lets you add devices which will show up in the EZlogic web UI.

loadfile("HUB:prefix.node_test_plugin/scripts/functions/create_device")()

--        Notes and guidelines for the startup.lua script:

-- * Keep this file as simple and as clean as possible. It should be readable and tell you
--   clearly what this plugin needs to do when it starts.

-- * Only write "business logic" code here that shows what this plugin needs to do when started

-- * Make sure you initialize all your data at plugin startup. For example, starting timers,
--   initializing global state, doing initial checks, initializing devices, starting discovery, etc.

-- * 'startup.lua' is loaded every time the firmware starts (or the ha-luad process is rebooted to be exact).
--   This can happen when a dependency crashes and the dependency tree is restarted, or when the hub is power-cycled,
--   rebooted or manually restarted. The latter often happens as plugin developers usually reboot to reload the config.json file when it contains changes.

-- * Note -  All plugins execute in the same (single) thread. A core aspect of our Lua API is that it is mostly asynchronous.
--   This lets plugins run quickly even if they all execute under a single thread. You'll notice that, for example,
--   when working with http requests or sockets, you have to subscribe to module events to continue processing the reponse.

-- * Documentation on config.json and interface.json structure is available at https://developer.mios.com/api/hub/plugins/

-- * Ezlo API concepts are as follows
--[[
    - GATEWAY - The plugin object. It describes the plugin using the information from config.json
    
      * You can check all gateways on a controller with the following websocket request 
      in our API tool at: https://apitool .ezlo.com/dashboard:
      
        {"id": "_AUTO_74107","method": "hub.gateways.list","params": {}}
        
      * The API tool lets you simulate the requests that are made by our mobile and web apps
      so you can test and debug your plugins.
        
      * The UI will be notified with the following broadcast every time you install a gateway:
      
        https://developer.mios.com/api/hub/broadcasts/hub-gateway-added/

        - [0..*] DEVICE - The virtual entity/container representing a physical device, service or component. 
        Basically anything you want.
        
         * Each gateway can have zero or more devices
         
         * Devices can have a parent-child relationship. For example, a thermostat device may contain 
         several sensors (motion,temperature, humidity etc) as child devices. 
         
         * We can represent this as a main device (the thermostat) and additional devices representing 
         each sensor which are mapped as children of the parent. 
         The advantage is that once you remove the parent, all child devices are also removed.
           
         * The UI will be notified with the following broadcast every time 
         you modify/add/remove a device inside the plugin:
         
         https://developer.mios.com/api/hub/broadcasts/hub-device-added/
           
         * The Lua device object is documented here: https://developer.mios.com/api/scripting/modules/core/objects/device/
         
         * APIs to work with devices from plugin code: https://developer.mios.com/api/scripting/modules/core/functionality/
         
         * APIs to work with devices from the API tool: https://developer.mios.com/api/hub/devices/api/


            - [0..*] ITEM - a device capability that we interact with when using a device. 
            For example, 'Start Recording' and 'Stop Recording' are capabilities (items) of a camera device.
            
              * Each device can have zero or more items (capabilities)
             
              * Items can participate in automations (scenes or meshbots).
              
              * Items we have already defined are available at: https://developer.mios.com/api/hub/items/item-names/items/
              The 'Enum' column links to allowed values/settings for the item.
                
              * Use the defined items in your plugin if you want the item to be visible in the UI. 
              Plugin items not shown on this list will not be visible in the UI.
                
              * Theoretically you can add any number of items to a device, but in the UI we only support rendering
                specific items for specific types of devices. For now, there is no support in the UI to render 
                any item dynamically.
                
              * The UI will be notified with the following broadcast each time 
              you modify/add/remove a device inside a plugin:
                
              https://developer.mios.com/api/hub/broadcasts/hub-item-added/ 
                
              * The item object is documented here: 
              
              https://developer.mios.com/api/scripting/modules/core/objects/item/
              
              * API's to work with items from plugin code: 
              
              https://developer.mios.com/api/scripting/modules/core/functionality/
              
              * API's to work with items from the API tool: https://developer.mios.com/api/hub/items/api/

            - [0..*] DEVICE SETTING - Settings are configuration values for device capabilities/items. 
            With gateway plugins, you can create custom settings for your logical/virtual devices.
            
              * Each device can have zero or more device settings.
              
              * Device settings behave like items but are a little different.
              
              * Device settings cannot participate in automations (scenes, meshbots).
                
              * Device settings can change the behavior of a device, but can also be used to add 
              new settings that are solely plugin managed.
              
              Example 1 - To let users change the polling frequency of each deviceâ€™s status.
              
              Example 2 - A setting to change the color saturation of a camera could be forwarded
              to the physical device via an API call, just as you would do with an item implementation. 
                
              * You can view available settings for each of your devices in the â€˜Devicesâ€™ section
              of the EZlogic portal - https://ezlogic.mios.com ('Settings > 'Devices')
                
              * With a few exceptions, our mobile apps do not currently show device settings in the UI.
                
              * The UI will be notified with the following broadcast each time you modify/add/remove
              a device setting inside a plugin: 
              
              https://developer.mios.com/api/hub/broadcasts/hub-device-setting-added/
                
              * The device setting object is documented here: 
              
              https://developer.mios.com/api/scripting/modules/core/objects/device-setting/
              
              * API's to work with device settings from plugin code:
              
              https://developer.mios.com/api/scripting/modules/core/functionality/add_setting/
              
              * API's to work with device settings from the API tool: 
              
              https://developer.mios.com/api/hub/devices/settings/api/
]]

-- -* We have additional API's not documented here because this is a "getting started" example.

--    Our documentation, available at https://developer.mios.com/api/, contains examples 
--    and explanations for all APIs and modules we make available for use in plugins. 

--    For example, under 'Scripting' > 'Modules', you can find the timer module: 
--    https://developer.mios.com/api/scripting/modules/timer/timer-module-description/

