-- The 'core' module provides the ability to work with the Ezlo Lua API to manage devices, items, settings and more.
-- Lua core module documentation is available at: https://developer.mios.com/api/scripting/modules/core/functionality/

local _core = require("core")

-- The 'storage' module provides the ability to store data that is specific to a plugin.
-- Each plugin has its own private storage in a key-value table.
-- This storage keeps a record of persistent private data like credentials, device IP address, MAC address, etc.

-- Lua storage module documentation is available at https://developer.mios.com/api/scripting/modules/storage/

local _storage = require("storage")

-- The 'logging' module handles log processing and supports different log severities, indents, etc.
-- Users can change the global log severity on our HUB and can modify log verbosity levels. 
-- More verbose logs are helpful when debugging issues.

-- The Logger API, which lets you change hub log level, is at https://developer.mios.com/api/hub/system/logging/hub-log-local-set/

-- Lua log module documentation is available at https://developer.mios.com/api/scripting/modules/logging/

local _logger = require("logging")

-- '_logger.info' will save the logs on the HUB at: /var/log/firmware/ha-luad.log
-- Example: 2021-12-13 19:32:11.863117 INFO : prefix.node_test_plugin: prefix.node_test_plugin teardown...
-- Lua logging module documentation is available at https://developer.mios.com/api/scripting/modules/logging/
-- It is best to log lifecycle events at 'info' severity level as this is the default severity for all controllers.

_logger.info("prefix.node_test_plugin teardown...")

-- Another good practice is to log a message before calling some functionality so that
-- you can trace the execution to the line that threw an exception/error

_logger.info("Cleanup storage...")

-- '_storage.delete_all()' clears all your stored data. It is always best to clear stored data 
-- when teardown is called so you don't leave uncleaned resources behind.

_storage.delete_all()

_logger.info("Cleanup devices...")

-- '_core.remove_gateway_devices()' deletes all devices created by the plugin.
-- NOTE - a 'plugin' is called a 'gateway' in controller API terminology, but they are the same thing.
-- The firmware will automatically remove all devices when a plugin is uninstalled, but it is still
-- good practice to call this method anyway. For one, it makes testing your plugin easier.

_core.remove_gateway_devices(_core.get_gateway().id)

--            NOTES and guidelines for the teardown.lua script

-- * Always clean any resources used *exclusively* by the plugin (timers, pending requests, opened sockets, temporary files, etc)

-- * I also recommend you manually delete devices created by your plugin as older firmware versions don't do it.
